clc
clear
close all

global xData yData yCalc glbRsqr QSPcoeff
zFilename = "besselj_0_x_halton_random_fx2";
txtFile = strcat(zFilename, ".txt");
xlFile = strcat(zFilename, ".xlsx");
diary(txtFile)
gFile =  strcat(zFilename, ".jpg");
fprintf("%s\n", datetime(now,'ConvertFrom','datenum'));
format longE
sEqn = "besselj(0,x)";
fprintf("%s\n",sEqn);
sEqnFx = "x^1.5/ln(x+1)";
fprintf("%s\n", sEqn);
fprintf("pwrFx = %s\n", sEqnFx);
fprintf("x=0:0.1:10\n")
xData= 0:0.1:10;
xData = xData';
n = length(xData);
yData = besselj(0,xData);
order = 6;
[Lb,Ub] = makeLimits(@(x)x^1.5./log(1+x), order, 0.1);
fprintf("       Lb              Ub\n");
fprintf("--------------------------------\n")
for i=1:order
  fprintf("    %f         %f\n", Lb(i), Ub(i))
end

[bestX,bestFx] = haltonRandomSearch(@quantShammasPoly,Lb,Ub,1000000);

SSE = quantShammasPoly(bestX);
% calculate adjusted value of the coefficient of determination
glbRsqr = 1 - (1 - glbRsqr)*(n-1)/(n-order-1);
fprintf("Adjusted Rsqr = %f\n", glbRsqr);
fprintf("Quantum Shammas Polynomial Powers\n");
bestX
fprintf("Quantum Shammas Polynomial Coefficients\n");
QSPcoeff = QSPcoeff'
fprintf("\nRegular polynomial fit\n");
c = polyfit(xData,yData,order)
yPoly = polyval(c,xData);
r = rsqr(yData,yPoly);
% calculate adjusted value of the coefficient of determination
r = 1 - (1 - r)*(n-1)/(n-order-1);
fprintf("Adjusted Rsqr = %f\n", r);
figure(1)
plot(xData,yData,xData,yCalc,xData,yPoly);
title(sEqn)
xlabel("X")
ylabel("Y");
grid;
ax = gca;
exportgraphics(ax,gFile);

QSPpwr = bestX;
Coeff = flip(c);
T = array2table(Lb);
writetable(T,xlFile,"Sheet","Sheet1","Range","A1");
T = array2table(Ub);
writetable(T,xlFile,"Sheet","Sheet1","Range","A4");
T1 = array2table(QSPpwr);
writetable(T1,xlFile,"Sheet","Sheet1","Range","A7");
T2 = array2table(QSPcoeff);
writetable(T2,xlFile,"Sheet","Sheet1","Range","A10");
T3 = array2table(Coeff);
writetable(T3,xlFile,"Sheet","Sheet1","Range","A13");
r_sqr = [glbRsqr r];
T4 = array2table(r_sqr);
writetable(T4,xlFile,"Sheet","Sheet1","Range","A16");

format short
diary off

function [Lb,Ub] = makeLimits(fx, order, gap)
  Lb = zeros(1,order);
  Ub = zeros(1,order);
  pwr = [];
  for i=1:order+1
    pwr =  [pwr fx(i)];
  end
  diff = pwr(2) - pwr(1);
  delta = (diff - gap)/2;
  Lb(1) = pwr(1) - delta;
  for i=2:order
    Ub(i-1) = pwr(i-1) + delta;
    diff = pwr(i+1) - pwr(i);
    delta = (diff - gap)/2;
    Lb(i) = pwr(i) - delta;
  end
  Ub(order) = pwr(order) + delta;
end

function r = rsqr(y,ycalc)
  n = length(y);
  ymean = mean(y);
  SStot = sum((y - ymean).^2);
  SSE = sum((y - ycalc).^2);
  r = 1 - SSE / SStot;
end

